@@ -22,6 +22,8 @@ our $RX_EXEC = $rx_expr;
 our $RX_ITEM = '\[(-?\d+)\]$';
 our $RX_SLCE = '\[(-?\d+)\s*\.\.\s*(-?\d+)\]$';
 
+our %CACHE;
+
 sub new {
     my ( $class, $node ) = @_;
     my $self = bless [ $node ], $class;
@@ -38,6 +40,8 @@ sub compile {
     $expr = ".$expr" unless substr( $expr, 0, 1 ) eq '.';
     die "failed to parse `$expr'" unless $expr =~ /$RX_TEST/g;
 
+    return @{ $CACHE{ $expr } } if exists $CACHE{ $expr };
+
     my @exec;
     my @tokens = $self->tokenize( $expr );
 
@@ -116,6 +120,8 @@ sub compile {
             push @exec, [ 'slice', 0+$start, 0+$end ];
         }
     }
+
+    $CACHE{ $expr } = [ @exec ];
     return @exec;
 }
 
@@ -153,8 +159,8 @@ sub parent {
 
 sub children {
     my ( $self, $node, $type ) = @_;
-    if ( UNIVERSAL::can( $node, 'children' ) and @{ $node->children } ) {
-        foreach my $child ( @$node ) {
+    if ( UNIVERSAL::can( $node, 'children' ) ) {
+        foreach my $child ( @{ $node->children } ) {
             next if defined $type and ( $child->type != $type );
             push @$self, $child;
         }
@@ -198,7 +204,12 @@ sub descendants {
             my $n = $stack[ $count ];
             if ( $name eq '*' and $n->type == ELEMENT_NODE ) {
                 push @$self, $n;
-            } elsif ( $n->name eq $name ) {
+            }
+            elsif ( $name eq '#processing-instruction' and
+                ( $n->type == PROCESSING_INSTRUCTION_NODE ) ) {
+                push @$self, $n;
+            }
+            elsif ( $n->name eq $name ) {
                 push @$self, $n;
             }
             if ( UNIVERSAL::can( $n, 'children' ) ) {
@@ -207,6 +218,7 @@ sub descendants {
                 }
             }
         }
+        undef( @stack );
     }
     $self;
 }
